/**
 * Copyright (c) 2009 Mark Logic Corporation. All rights reserved.
 */
package com.marklogic.cpox;

/**
 * @author Michael Blakeley, Mark Logic Corporation
 *
 */
public class TimedEvent {
    
    private static final String DEFAULT_DESCRIPTION = "(n/a)";

    private long bytes = 0;

    private long duration = -1;

    private boolean error = false;

    private long start;
    
    private String description = null;

    public TimedEvent() {
        start = System.nanoTime();
        description = DEFAULT_DESCRIPTION;
    }

    public TimedEvent(boolean _error) {
        error = _error;
        start = System.nanoTime();
    }

    /**
     * @return
     */
    public long getBytes() {
        return bytes;
    }

    /**
     * 
     */
    public long stop() {
        return stop(bytes < 0 ? 0 : bytes, false);
    }

    public long stop(long _bytes) {
        return stop(_bytes, false);
    }

    /**
     * @param _error
     */
    public void stop(boolean _error) {
        stop(-1, _error);
    }

    public long stop(long _bytes, boolean _error) {
        // duplicate calls to stop() should be harmless
        if (duration > -1) {
            return duration;
        }

        duration = System.nanoTime() - start;
        // timings of 0 are very bad for averaging,
        // but is the cure worse than the disease?
        assert duration > 0;
        // alternative idea:
        //if (duration < 1) {
        //duration = 1;
        //}

        // bytes == -1 is a flag
        if (_bytes > -1) {
            bytes = _bytes;
        }
        error = _error;
        return duration;
    }

    /**
     * @return
     */
    public long getDuration() {
        if (duration < 0) {
            return System.nanoTime() - start;
        }
        
        return duration;
    }

    /**
     * @return
     */
    public boolean isError() {
        return error;
    }

    /**
     * @return
     */
    public long getStart() {
        return start;
    }

    public void increment(long _bytes) {
        bytes += _bytes;
    }

    /**
     * @param b
     */
    public void setError(boolean b) {
        error = true;
    }

    public String getDescription() {
        return description;
    }

    public void setDescription(String description) {
        this.description = description;
    }

}
