/**
 * Copyright (c) 2009-2010 Mark Logic Corporation. All rights reserved.
 */
package com.marklogic.cpox;

import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.net.URI;
import java.net.URISyntaxException;
import java.security.KeyManagementException;
import java.security.NoSuchAlgorithmException;
import java.security.cert.CertificateException;
import java.security.cert.X509Certificate;
import java.util.Iterator;
import java.util.Map;

import javax.net.ssl.SSLContext;
import javax.net.ssl.TrustManager;
import javax.net.ssl.X509TrustManager;

import com.marklogic.xcc.ContentSource;
import com.marklogic.xcc.ContentSourceFactory;
import com.marklogic.xcc.Request;
import com.marklogic.xcc.ResultSequence;
import com.marklogic.xcc.SecurityOptions;
import com.marklogic.xcc.Session;
import com.marklogic.xcc.exceptions.XccConfigException;
import com.marklogic.xcc.exceptions.XccException;

/**
 * @author Michael Blakeley, Mark Logic Corporation
 * 
 */
public class XccInvokeWorker extends Worker {

    protected String connectionString;

    protected ContentSource cs;

    protected Session session;

    protected Request request;

    protected SecurityOptions securityOptions;

    protected boolean isSecure = false;

    // SSL support
    protected static SecurityOptions newTrustAnyoneOptions()
            throws KeyManagementException, NoSuchAlgorithmException {
        TrustManager[] trust = new TrustManager[] { new X509TrustManager() {
            public java.security.cert.X509Certificate[] getAcceptedIssuers() {
                return new X509Certificate[0];
            }

            /**
             * @throws CertificateException
             */
            public void checkClientTrusted(
                    java.security.cert.X509Certificate[] certs,
                    String authType) throws CertificateException {
                // no exception means it's okay
            }

            /**
             * @throws CertificateException
             */
            public void checkServerTrusted(
                    java.security.cert.X509Certificate[] certs,
                    String authType) throws CertificateException {
                // no exception means it's okay
            }
        } };

        SSLContext sslContext = SSLContext.getInstance("SSLv3");
        sslContext.init(null, trust, null);
        return new SecurityOptions(sslContext);
    }

    /**
     * @param controller
     * @param index
     */
    public XccInvokeWorker(Controller controller, int index) {
        super(controller, index);

        connectionString = config.getConnectionString();

        try {
            URI uri = new URI(connectionString);

            // SSL support
            String scheme = uri.getScheme();
            isSecure = "https".equals(scheme) || "xccs".equals(scheme);
            if (isSecure) {
                securityOptions = newTrustAnyoneOptions();
            }

            cs = isSecure ? ContentSourceFactory.newContentSource(uri,
                    securityOptions) : ContentSourceFactory
                    .newContentSource(uri);
            session = cs.newSession();
        } catch (XccConfigException e) {
            throw new FatalException(connectionString, e);
        } catch (URISyntaxException e) {
            throw new FatalException(connectionString, e);
        } catch (KeyManagementException e) {
            throw new FatalException(connectionString, e);
        } catch (NoSuchAlgorithmException e) {
            throw new FatalException(connectionString, e);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.marklogic.cpox.Worker#bookmarkAction(java.lang.String)
     */
    @Override
    protected InputStream bookmarkAction(String user, String id)
            throws ActionException {
        request = session.newModuleInvoke("bookmark.xqy");
        request.setNewStringVariable("USER-ID", user);
        request.setNewStringVariable("DOC-ID", id);
        return doRequest();
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.marklogic.cpox.Worker#searchAction(java.lang.String[],
     * java.lang.String, int, java.util.Map)
     */
    @Override
    protected InputStream searchAction(String[] query, String language,
            int start, Map<String, String> facets) throws ActionException {
        request = session.newModuleInvoke("search.xqy");
        request.setNewIntegerVariable("START", start);
        request.setNewStringVariable("LANGUAGE", language);
        StringBuilder queryString = new StringBuilder(Utilities.join(
                query, " "));
        // add any facet parameters
        if (!facets.isEmpty()) {
            Iterator<String> keys = facets.keySet().iterator();
            String key = null;
            String value = null;
            while (keys.hasNext()) {
                key = keys.next();
                value = facets.get(key);
                queryString.append(" ").append(key).append(":\"").append(
                        value).append("\"");
            }
        }
        request.setNewStringVariable("QUERY", queryString.toString());
        return doRequest();
    }

    /**
     * @return
     * @throws ActionException
     */
    private InputStream doRequest() throws ActionException {
        try {
            ResultSequence rs = session.submitRequest(request);
            if (null == rs) {
                return null;
            }
            if (!rs.hasNext()) {
                return new ByteArrayInputStream(new byte[0]);
            }
            return rs.next().asInputStream();
        } catch (XccException e) {
            throw new ActionException(e);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.marklogic.cpox.Worker#updateAction(java.lang.String,
     * java.lang.String)
     */
    @Override
    protected InputStream updateAction(String id, String token)
            throws ActionException {
        request = session.newModuleInvoke("update.xqy");
        request.setNewStringVariable("ID", id);
        request.setNewStringVariable("TOKEN", token);
        return doRequest();
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.marklogic.cpox.Worker#verifyAction(java.lang.String)
     */
    @Override
    protected InputStream verifyAction(String token)
            throws ActionException {
        request = session.newModuleInvoke("verify.xqy");
        request.setNewStringVariable("TOKEN", token);
        return doRequest();
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.marklogic.cpox.Worker#viewAction(java.lang.String)
     */
    @Override
    protected InputStream viewAction(String id) throws ActionException {
        request = session.newModuleInvoke("view.xqy");
        request.setNewStringVariable("ID", id);
        return doRequest();
    }

}
